cordova.define("com.holenderski.utils.Utils", function(require, exports, module) {
'use strict';

var Utils = (function () {

  /*
   * Dump all fields of an object.
   */

  function dump(obj, indent, out) {

    if (typeof obj === "string") {
      return obj;
    }

    indent = indent || "  ";
    out = out || "";

    for (var key in obj) {
      var value = obj[key];
      if (value && typeof value === "object") {
        // handle date object differently, see http://stackoverflow.com/questions/643782
        if (Object.prototype.toString.call(value) === '[object Date]') {
          out += indent + key + ": " + value + "\n";
        } else {
          out += indent + key + ": \n";
          out += Utils.dump(value, indent + "  ");
        }

      } else {
        out += indent + key + ": " + value + "\n";
      }
    };
    return out;
  }

  function merge(obj1, obj2) {
    for (var attrname in obj2) { obj1[attrname] = obj2[attrname]; }
    return obj1;
  }

  function numberList(id, min, max, options) {
    options = merge({ padding: 0, value: undefined }, options);

    var html = '<select id="' + id + '" class="detailDateList">';
    for (var i = min; i <= max; i++) {
      var selected = options.value === i ? "selected" : "";
      html += '<option value="' + i + '" ' + selected + '>' + (options.padding ? pad(i, options.padding) : i) + "</option>";
    }
    html += '</select>';

    return html;
  }

  function optionList(id, list, options) {
    options = merge({ value: undefined }, options);

    var html = '<select id="' + id + '">';
    list.forEach(function (item) {
      var selected = options.value !== undefined && options.value == item.key ? "selected" : "";
      html += '<option value="' + item.key + '" ' + selected + '>' + item.value + "</option>";
    });
    html += '</select>';
    return html;
  }

  var months = [
    { key: 1, value: "Januari" },
    { key: 2, value: "Februari" },
    { key: 3, value: "Maart" },
    { key: 4, value: "April" },
    { key: 5, value: "Mei" },
    { key: 6, value: "Juni" },
    { key: 7, value: "Juli" },
    { key: 8, value: "Augustus" },
    { key: 9, value: "September" },
    { key: 10, value: "Oktober" },
    { key: 11, value: "November" },
    { key: 12, value: "December" }
  ];

  /*
   * Load a page
   */

  function getFile(path, options) {
    if (!path) { throw "path == null"; }

    options = merge({
      type: "text", // can be "text", "xml", or "json"
      silent: false
    }, options);

    var result = undefined;
    try {
      // prepend the docRoot
      if (Global.platform === "ios" || Global.platform === "android" || Global.platform === "windows8" || Global.platform === "windows7") {
        if (Global.docRootPath === undefined) {
          throw "Global.docRootPath undefined";
        }
        path = Global.docRootPath + path;
      }

      $.ajax({
        url: path,
        success: function (html) {
          result = html;
        },
        error: function (error) {
          alert("file not found '" + path + "(possibly empty file)", "getFile()");
        },
        dataType: options.type,
        async: false
      });
    } catch (e) {
      if (!options.silent) { alert(e, "getFile()"); }
      return null;
    }

    return result;
  }

  function pad(num, size) {
    var s = num + "";
    while (s.length < size) s = "0" + s;
    return s;
  }

  function round(num, decimals) {
    var tens = Math.pow(10, decimals);
    return Math.round(num*tens)/tens;
  }

  function indexOf(array, field, value) {
    for (var i = 0; i < array.length; i++) {
      if (array[i][field] == value) return i;
    }
    return -1;
  }

  function find(array, field, value) {
    var index = indexOf(array, field, value);
    if (index > -1) {
      return array[index];
    } else {
      return undefined;
    }
  }

  function splice(array, field, value, replacement) {
    var index = indexOf(array, field, value);
    if (index > -1) {
      if (replacement) {
        return array.splice(index, 1, replacement);
      } else {
        return array.splice(index, 1);
      }
    } else {
      return undefined;
    }
  }

  /*
   * Clone an object
   *
   * Taken from http://stackoverflow.com/a/728694
   */

  function clone(obj) {
    // Handle the 3 simple types, and null or undefined
    if (null == obj || "object" != typeof obj) return obj;

    // Handle Date
    if (obj instanceof Date) {
      var copy = new Date();
      copy.setTime(obj.getTime());
      return copy;
    }

    // Handle Array
    if (obj instanceof Array) {
      var copy = [];
      for (var i = 0, len = obj.length; i < len; i++) {
        copy[i] = clone(obj[i]);
      }
      return copy;
    }

    // Handle Object
    if (obj instanceof Object) {
      var copy = {};
      for (var attr in obj) {
        if (obj.hasOwnProperty(attr)) copy[attr] = clone(obj[attr]);
      }
      return copy;
    }

    throw new Error("Unable to clone object. Its type isn't supported.");
  }

  function values(obj) {
    var values = [];
    for (var key in obj) {
      values.push(obj[key]);
    }
    return values;
  }

  /*
   * Return string looking like "28/03/2013 15:00:07"
   */

  function dateToCommonString(d, options) {
    options = Utils.merge({
      mode: "long",
      seconds: false
    }, options);

    if (!d) { return ""; }

    var year = d.getFullYear();
    var month = Utils.pad(d.getMonth() + 1, 2);
    var day = Utils.pad(d.getDate(), 2);
    var hour = Utils.pad(d.getHours(), 2);
    var minute = Utils.pad(d.getMinutes(), 2);
    var seconds = Utils.pad(d.getSeconds(), 2);

    if (options.mode == "time") {
      return hour + ":" + minute;

    } else if (options.mode == "short") {
      return day + "." + month + "." + year;
    
    } else {
      if (options.seconds) {
        return day + "." + month + "." + year + " " + hour + ":" + minute + ":" + seconds;
      } else {
        return day + "." + month + "." + year + " " + hour + ":" + minute;
      }
    }
  }

  function millisecondsToString(milliseconds) {
    var seconds = Math.floor(milliseconds / 1000) % 60;
    var minutes = Math.floor(milliseconds / (1000 * 60)) % 60;
    var hours = Math.floor(milliseconds / (1000 * 60 * 60));
    return hours + ":" + Utils.pad(minutes, 2) + ":" + Utils.pad(seconds, 2);
  }

  /*
   * Comptue distance between two GPS coordinates.
   * Taken from: http://www.movable-type.co.uk/scripts/latlong.html
   */

  function distance(coord1, coord2) {
    if (coord1 === undefined || coord2 === undefined
    ||  coord1.lat === undefined || coord1.lon === undefined
    ||  coord2.lat === undefined || coord2.lon === undefined
    ) {
      return undefined;
    }
  
    var R = 6371; // km
    var dLat = (coord2.lat - coord1.lat).toRad();
    var dLon = (coord2.lon - coord1.lon).toRad();
    var lat1 = coord1.lat.toRad();
    var lat2 = coord2.lat.toRad();

    var a = Math.sin(dLat / 2) * Math.sin(dLat / 2) +
            Math.sin(dLon / 2) * Math.sin(dLon / 2) * Math.cos(lat1) * Math.cos(lat2);
    var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
    var d = R * c;

    return d;
  }

  /*
   * Color between a and b with 0 <= d <= 1 being the ratio between the two.
   * Both colors a and b are triples { r: x, g: y, b: z}, 
   * with x, y, z being integers between 0 and 255.
   */

  function colorBetween(a, b, d) {
    function f(x, y, u) {
      return Math.max(0, Math.min(255, Math.round((1 - u) * x + u * y)));
    }
    return {
      r: f(a.r, b.r, d),
      g: f(a.g, b.g, d),
      b: f(a.b, b.b, d)
    }
  }

  /*
   * Localization support
   */

  var _lang = null;

  function setLang(lang) {
    _lang = lang;
  }

  function addLang(lang) {
    _lang = _lang || {};
    setLang(merge(_lang, lang));
  }

  function localize(html) {
    var key = null;
    while (key = /{(.*?)}/.exec(html)) {
      key = key[1];
      html = html.replaceSafe(new RegExp("{" + key + "}"), localizeSingle(key));
    }
    return html;
  }
  
  function localizeSingle(key, args) { 
    var text = _lang[key];
    if (!text) {
      alert("Unknown lang key '" + key + "'");
      throw new Error("Unknown lang key '" + key + "'");
    }

    // make any substitutions. Note that variable arguments start at index 1, but the 
    // format string will start indexing substitutions at 0.
    for (var i = 1; i < arguments.length; i++) {
      text = text.replaceSafe(new RegExp("\\{" + (i-1) + "\\}", "g"), arguments[i]);
    }
    return text;
  };

  function langExists(key) {
    return _lang[key] ? true : false;
  }

  /* words: [rok, lata, lat] */
  /* words: [roza, roze, roz] */
  function plural(number, words) {
    if (number == 1) {
      return words[0];
    } else if (2 <= number && number <= 4) {
      return words[1];
    } else if (5 <= number && number <= 19) {
      return words[2];
    } else if (number % 10 == 2 || number % 10 == 3 || number % 10 == 4) {
      return words[1];
    } else {
      return words[2];
    }
  }

  function networkErrorAlert(error) {
    if (!error || error == "network_error") {
      info(localizeSingle("network_error"), { tag: "network_error" });
    } else {
      info(localizeSingle(error), { tag: "network_error" });
    }
  }

  function networkErrorScreen(error, screen, refreshButton) {
    if (error != "network_error") return;

    var button = refreshButton
      ? {
        title: _("refresh"),
        handler: function () {
          screen.refresh({fetch: true});
        }
      }
      : undefined

    screen.showEmptyContent("networkError", "networkError", localizeSingle("network_error"), button);
  }

  function checkStatus(data, success, fail) {
    if (data.status === "ok") {
      success();
    } else {
      switch (data.reason) {
        case "wrong_user_or_password": 
        case "missing_user":
        case "missing_password": 
        case "email_exists":
        case "user_blocked":
          info(localizeSingle(data.reason));
          break;
        case "server_error":
          info(localizeSingle(data.reason), { hideDelay: 0 });
          break;
        default: info(localizeSingle("unknown_error", data.reason));
      }
      fail();
    } 
  }
 
  function append(node, html) {
    node.append(html);
  }

  function appendTo(html, node) {
    return $(html).appendTo(node);
  }

  function prepend(node, html) {
    node.prepend(html);
  }

  function after(node, html) {
    node.after(html);
  }

  /*
  Not used anymore and gives a warning in Google Closure Compiler.
  function forceCSSRefresh(node) {
    if (node && node[0]) {
      var sel = node[0];
      var display = sel.style.display;
      sel.style.display = 'none';
      sel.offsetHeight; // no need to store this anywhere, the reference is enough
      sel.style.display = display;
    } else {
      debug("trying to forceCSSRefresh on an undefined");
    }
  }
  */
  
  /*
   * Read less variables in javascript.
   * Taken from https://gist.github.com/Sjeiti/2948738

       #foobar {
        .number0 { width: 1234px; }
        .number1 { width: 1234em; }
        .number2 { width: 5.5%; }
        .number3 { opacity: .123; }
        .string0 { width: auto; }
        .string1 { position: absolute; }
        .string2 { font: Arial; }
        .color0 { color: #800000; }
        .color1 { color: #666; }
        .color2 { color: rgb(100, 200, 50 ); }
        .color3 { color: rgba(100, 200, 50, 0.4 ); }
        .content0 { content: 'a string is a string'; }
        .content1 { content: a string is a string; }
        .content2 { content: "a string is a string"; }
        .content3 { content: rgba(100, 200, 50, 0.4 ); }
        .content4 { content: 125px; }
        .content5 { content: #666; }
        .content6 { content: attr(title) "qux"; }
        .content7 { content: '#666'; }
        .identifier0 { asdf: 125px; }
        .identifier1 { -foo-bar: 125px; }
        .uri0 { background: url(http://foo.com/bar.png); }
        .uri1 { background-image: url(http://foo.com/bar.png); }
        .uri2 { content: url(http://foo.com/bar.png); }
    }
  */

  function getLessVars(id, parseNumbers) {
    var bNumbers = parseNumbers === undefined ? true : parseNumbers
      , oLess = {}
      , rgId = /\#\w+/
      , rgKey = /\.(\w+)/
      , rgUnit = /[a-z]+$/
      , aUnits = 'em,ex,ch,rem,vw,vh,vmin,cm,mm,in,pt,pc,px,deg,grad,rad,turn,s,ms,Hz,kHz,dpi,dpcm,dppx'.split(',')
      , rgValue = /:\s?(.*)\s?;\s?\}/
      , rgStr = /^'([^']+)'$/
      , sId = '#' + id
      , oStyles = document.styleSheets;
    for (var i = 0, l = oStyles.length; i < l; i++) {
      var oRules = oStyles[i].cssRules;
      for (var j = 0, k = oRules.length; j < k; j++) {
        var sRule = oRules[j].cssText
          , aMatchId = sRule.match(rgId);
        if (aMatchId && aMatchId[0] == sId) {
          var aKey = sRule.match(rgKey)
            , aVal = sRule.match(rgValue);
          if (aKey && aVal) {
            var sKey = aKey[1]
              , oVal = aVal[1]
              , aUnit
              , aStr;
            if (bNumbers && (aUnit = oVal.match(rgUnit)) && aUnits.indexOf(aUnit[0]) !== -1) {
              oVal = parseFloat(oVal);
            } else if (aStr = oVal.match(rgStr)) {
              oVal = aStr[1];
            }
            oLess[sKey] = oVal;
          }
        }
      }
    }
    return oLess;
  }
  
  var getLessVar = function (name, prop) {
    var node = $('<div class="' + name + '"></div>').hide().appendTo('body');
    var value = node.css(prop);
    node.remove();
    return /^\d+/.test(value) ? +value : value
  }

  function printFileErrors() {
    for (var key in FileError) {
      debug("FileError." + key + ": " + FileError[key]);
    }
  }

  function printFileTransferErrors() {
    for (var key in FileError) {
      debug("FileTransferError." + key + ": " + FileTransferError[key]);
    }
  }
  
  /*
   * Hide the software keyboard (on iOS)
   * Taken from http://petercorvers.nl/phonegapcordova-hide-ios-keyboard
   */
   
  function hideKeyboard() {
    //debug("Utils.hideKeyboard");
    
    // DISABLE bluring because otherwise it interferes e.g. with blur handlers on the form
    //document.activeElement.blur();
    //$("input").blur();
  }

  function addActiveHighlight(node, selector) {
    // highlight the selected option
    node.find(selector).hammer().on("touch", function () {
      $(this).addClass("active");
    });
    node.find(selector).hammer().on("release", function () {
      $(this).removeClass("active");
    });
  }

  /*
   * Base64 encoding.
   * Taken from http://www.sumitgupta.net/javascript-base64-encode-decode-for-utf-8unicode-string/
   *
   * Note: the jQuery pligin does not work with UTF-8
   */

  function utf8_to_b64( str ) {
    return window.btoa(unescape(encodeURIComponent( str )));
  }
   
  function b64_to_utf8( str ) {
    return decodeURIComponent(escape(window.atob( str )));
  }

  /*
   * Playing sounds
   */

  function playAudio(url) {
    if (!Global || !Global.settings || Global.settings.sounds != "yes") return;
    
    //debug("play: " + url)
    // play one sound at a time
    //if (playAudio.playing) return;
    playAudio.playing = true;

    // Fix for android
    // taken from http://www.raymondcamden.com/2014/06/23/Cordova-Media-API-Example
    if (device.platform.toLowerCase() === "android") {
      url = "/android_asset/www/" + url;
    }

    // cache the loaded sounds
    playAudio.sounds = playAudio.sounds || {};
    if (!playAudio.sounds[url]) {
      playAudio.sounds[url] = new Media(url,
        // success callback
        function () {
          playAudio.playing = false;
        },
        // error callback
        function (err) {
          playAudio.playing = false;
          playAudio.sounds[url] = undefined;
          alert("Audio Error: " + Utils.dump(err) + "\nMediaErrors: " + Utils.dump(MediaError));
        }
      );
    } else {
      //debug("playing from cache")
    }

    // set playAudioWhenScreenIsLocked=false to respect the mute setting on the phone
    var sound = playAudio.sounds[url];
    //console.log("PLAYING");
    sound.play({ playAudioWhenScreenIsLocked : false });
  }

  // taken from http://stackoverflow.com/a/3627747
  function rgb2hex(rgb) {
    if (/^#[0-9A-F]{6}$/i.test(rgb)) return rgb;

    rgb = rgb.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
    function hex(x) {
        return ("0" + parseInt(x).toString(16)).slice(-2);
    }
    return "#" + hex(rgb[1]) + hex(rgb[2]) + hex(rgb[3]);
  }

  // export public methods
  var my = {};
  my.dump = dump;
  my.merge = merge;
  my.numberList = numberList;
  my.optionList = optionList;
  my.months = months;
  my.pad = pad;
  my.round = round;
  my.find = find;
  my.indexOf = indexOf;
  my.splice = splice;
  my.getFile = getFile;
  my.clone = clone;
  my.values = values;
  my.dateToCommonString = dateToCommonString;
  my.millisecondsToString = millisecondsToString;
  my.distance = distance;
  my.colorBetween = colorBetween;
  my.setLang = setLang;
  my.addLang = addLang;
  my.localize = localize;
  my._ = localizeSingle;
  my.langExists = langExists;
  my.plural = plural;
  my.networkErrorAlert = networkErrorAlert;
  my.networkErrorScreen = networkErrorScreen;
  my.checkStatus = checkStatus;
  my.append = append;
  my.appendTo = appendTo;
  my.prepend = prepend;
  my.after = after;
  my.getLessVars = getLessVars;
  my.getLessVar = getLessVar;
  my.printFileErrors = printFileErrors;
  my.printFileTransferErrors = printFileTransferErrors;
  my.hideKeyboard = hideKeyboard;
  my.addActiveHighlight = addActiveHighlight;
  my.utf8_to_b64 = utf8_to_b64;
  my.b64_to_utf8 = b64_to_utf8;
  my.playAudio = playAudio;
  my.rgb2hex = rgb2hex;
  return my;
}());

if (typeof module != 'undefined' && module.exports) {
    module.exports = Utils;
} else if (typeof window != 'undefined')  {
    window[exportName] = Utils;
}
});
